@@ -14,7 +14,7 @@ INTERACTION = 'INTERACTION'
 
 
 class MLDL:
-    def __init__(self, mongo_url, cache_size=10, sample_max_worker=16):
+    def __init__(self, mongo_url, cache_size=10):
         random.seed(time.time())
         self.client = MongoClient(mongo_url)
         self.db = self.client['A310']
@@ -38,8 +38,8 @@ class MLDL:
         self.kill_event = Event()
         self.kill_event_cache = Event()
         self.filler_thread_cache = None
-        self.sample_queue = Queue()
-        self.sample_max_worker = sample_max_worker
+        # self.sample_queue = Queue()
+        # self.sample_max_worker = sample_max_worker
 
         # Cache the data with regard to length of sequence
         tmp = self.db['length_freq_lang5_train'].find().sort('len', ASCENDING)
@@ -92,13 +92,21 @@ class MLDL:
             factor = 10
             if num < 1000:
                 factor = 10000 // num
+            start = time.perf_counter()
             c = col[MAIN].find({'len': {'$lt': max_length + 1}, 'token_size': {'$gt': min_num_feature - 1}},
-                               {'row_id': 1, '_id': 0}).sort("rand").limit(factor * num)
+                               {'row_id': 1, '_id': 0, 'rand': 1}, batch_size=1000).sort("rand", -1).limit(factor * num)
+            print(f'Query time: {time.perf_counter() - start}')
+            start = time.perf_counter()
             tmp = list(c)
+            print(f'Convert time: {time.perf_counter() - start}')
+            print(tmp[0], tmp[-1])
             # print(len(tmp))
+            start = time.perf_counter()
             self.sample_cache = list(map(lambda x: x['row_id'], tmp))
+            print(f'Convert time 2: {time.perf_counter() - start}')
+
             self.filler_thread_cache = Thread(target=self.background_sample_cache,
-                                              args=(max_length, num, min_num_feature, col[MAIN]))
+                                              args=(max_length, num, min_num_feature, col[MAIN], tmp[-1]['rand']))
             self.filler_thread_cache.start()
         else:
             self.sample_cache = range(1, maxi)
@@ -167,95 +175,111 @@ class MLDL:
         cursor = col.find({'row_id': {'$in': index_list}}, {'_id': 0, 'row_id': 1, 'interactions': 1})
         return pd.DataFrame(list(cursor))
 
-    def background_sample_cache(self, max_length, num, min_num_feature, col):
+    def background_sample_cache(self, max_length, num, min_num_feature, col, last_rand):
         """
         This function is used to increase the sample cache (suitable row_ids) in the background
         :param max_length: max length of sequence
         :param num: number of samples
         :param min_num_feature: min number of features
         :param col: main collection
+        :param last_rand: last rand value
         :return: extend the sample cache
         """
-        # print('starting the background sample cache')
-        max_workers = self.sample_max_worker
-        kill_event = Event()
-        i = 10
-        if num < 1000:
-            i = 10000 // num
-        listener = concurrent.futures.ThreadPoolExecutor()
-        # print('starting the listener')
-        listener_thread = listener.submit(self.background_sample_cache_listener, kill_event, max_workers)
-
-        # print('starting the executor')
-
-        executor = BoundedThreadPoolExecutor(max_workers=max_workers)
-        while i > 0 and not kill_event.is_set():
-            # print(f'getting the sample cache {i}')
-            executor.submit(self.background_sample_cache_filler, max_length, num, min_num_feature, col, i)
-            i += 1
-
-        # print('shutting down the executor')
-        listener_thread.result()
-        executor.shutdown(wait=True, cancel_futures=True)
-
-        # while i > 0:
-        #     c = col.find({'len': {'$lt': max_length + 1}, 'token_size': {'$gt': min_num_feature - 1}},
-        #                  {'row_id': 1, '_id': 0}).sort("rand").skip(i * num).limit(num)
-        #     tmp = list(c)
-        #     self.sample_cache += list(map(lambda x: x['row_id'], tmp))
-        #     i += 1
-        #     if len(tmp) < num or self.kill_event_cache.is_set():
-        #         i = 0
-        #         break
-
-    def background_sample_cache_filler(self, max_length, num, min_num_feature, col, i):
-        """
-        This function is used to increase the sample cache (suitable row_ids) in the background
-        :param max_length: max length of sequence
-        :param num: number of samples
-        :param min_num_feature: min number of features
-        :param col: main collection
-        :param i: index of the skip
-        :return: extend the sample cache
-        """
-        c = col.find({'len': {'$lt': max_length + 1}, 'token_size': {'$gt': min_num_feature - 1}},
-                     {'row_id': 1, '_id': 0}).sort("rand").skip(i * num).limit(num)
-        tmp = list(c)
-        tmp = list(map(lambda x: x['row_id'], tmp))
-        if self.kill_event_cache.is_set():
-            self.sample_queue.put([])
-            return
-
-        self.sample_queue.put(tmp)
-        if len(tmp) < num:
-            self.sample_queue.put([])
-
-    def background_sample_cache_listener(self, kill_event, max_workers):
-        i = 0
+        max_rand = last_rand
+        num = max(num, 1000)
         while True:
-            try:
-                tmp = self.sample_queue.get(timeout=3 * 60)
-                # print(f'get sample cache with len {len(tmp)}')
-                if self.kill_event_cache.is_set():
-                    self.sample_queue = Queue()
-                    kill_event.set()
-                    return
-                if len(tmp) == 0:
-                    # print('sample cache is empty')
-                    kill_event.set()
-                    i += 1
+            print('filling the cache')
+            c = col.find(
+                {'len': {'$lt': max_length + 1}, 'token_size': {'$gt': min_num_feature - 1}, 'rand': {'$lt': max_rand}},
+                {'row_id': 1, '_id': 0, 'rand': 1}, batch_size=1000).sort("rand", -1).limit(num)
+            tmp = list(c)
+            max_rand = tmp[-1]['rand']
+            self.sample_cache += list(map(lambda x: x['row_id'], tmp))
+            if len(tmp) < num or self.kill_event_cache.is_set():
+                break
 
-                self.sample_cache += tmp
+    #     # print('starting the background sample cache')
+    #     max_workers = self.sample_max_worker
+    #     kill_event = Event()
+    #     i = 10
+    #     if num < 1000:
+    #         i = 10000 // num
+    #     listener = concurrent.futures.ThreadPoolExecutor()
+    #     # print('starting the listener')
+    #     listener_thread = listener.submit(self.background_sample_cache_listener, kill_event, max_workers)
 
-                if i >= max_workers:
-                    # print('workers are all done')
-                    return
+    #     # print('starting the executor')
 
-            except Exception as error:
-                # print('time out on get sample cache')
-                kill_event.set()
-                self.sample_queue = Queue()
-                return
+    #     executor = BoundedThreadPoolExecutor(max_workers=max_workers)
+    #     while i > 0 and not kill_event.is_set():
+    #         # print(f'getting the sample cache {i}')
+    #         executor.submit(self.background_sample_cache_filler, max_length, num, min_num_feature, col, i)
+    #         i += 1
+
+    #     # print('shutting down the executor')
+    #     listener_thread.result()
+    #     executor.shutdown(wait=True, cancel_futures=True)
+
+    #     # while i > 0:
+    #     #     c = col.find({'len': {'$lt': max_length + 1}, 'token_size': {'$gt': min_num_feature - 1}},
+    #     #                  {'row_id': 1, '_id': 0}).sort("rand").skip(i * num).limit(num)
+    #     #     tmp = list(c)
+    #     #     self.sample_cache += list(map(lambda x: x['row_id'], tmp))
+    #     #     i += 1
+    #     #     if len(tmp) < num or self.kill_event_cache.is_set():
+    #     #         i = 0
+    #     #         break
+
+    # def background_sample_cache_filler(self, max_length, num, min_num_feature, col, i, min_rand, max_rand):
+    #     """
+    #     This function is used to increase the sample cache (suitable row_ids) in the background
+    #     :param max_length: max length of sequence
+    #     :param num: number of samples
+    #     :param min_num_feature: min number of features
+    #     :param col: main collection
+    #     :param i: index of the skip
+    #     :param last_rand: last rand value
+    #     :return: extend the sample cache
+    #     """
+    #     min_rand = min_rand
+    #     c = col.find({'len': {'$lt': max_length + 1}, 'token_size': {'$gt': min_num_feature - 1}, 'rand': {'$gt': min_rand}'},
+    #                  {'row_id': 1, '_id': 0}).sort("rand").skip(i * num).limit(num)
+    #     tmp = list(c)
+    #     tmp = list(map(lambda x: x['row_id'], tmp))
+    #     if self.kill_event_cache.is_set():
+    #         self.sample_queue.put([])
+    #         return
+
+    #     self.sample_queue.put(tmp)
+    #     if len(tmp) < num:
+    #         self.sample_queue.put([])
+
+    # def background_sample_cache_listener(self, kill_event, max_workers):
+    #     i = 0
+    #     while True:
+    #         try:
+    #             tmp = self.sample_queue.get(timeout=3 * 60)
+    #             # print(f'get sample cache with len {len(tmp)}')
+    #             if self.kill_event_cache.is_set():
+    #                 self.sample_queue = Queue()
+    #                 kill_event.set()
+    #                 return
+    #             if len(tmp) == 0:
+    #                 # print('sample cache is empty')
+    #                 kill_event.set()
+    #                 i += 1
+
+    #             self.sample_cache += tmp
+
+    #             if i >= max_workers:
+    #                 # print('workers are all done')
+    #                 return
+
+    #         except Exception as error:
+    #             # print('time out on get sample cache')
+    #             kill_event.set()
+    #             self.sample_queue = Queue()
+    #             return
 
     def terminate(self):
         """
